﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Editor.Document
{
    public sealed class DocumentSelection : ISelection
    {
        private Brush backColor;
        private EventArgs changeArgs;
        private bool objectChanging;
        private SSelection selection;
        private Color textColor;

        public Brush BackColor
        {
            get { return backColor; }
        }

        public bool Changing 
        {
            get { return objectChanging; }
        }

        public int EndIndex
        {
            get { return selection.EndIndex; }
            set 
            { 
                selection.EndIndex = value;

                ObjectChangedEvent( this, EventArgs.Empty );
            }
        }

        public int EndRow
        {
            get { return selection.EndRow; }
            set 
            { 
                selection.EndRow = value;

                ObjectChangedEvent( this, EventArgs.Empty );
            }
        }

        public bool MultiIndex 
        {
            get { return selection.MultiIndex; }
        }
        public bool MultiRow 
        {
            get { return selection.MultiRow; }
        }

        public bool Reversed 
        {
            get 
            {
                if( selection.StartRow == selection.EndRow ) 
                    return selection.StartIndex > selection.EndIndex;
                else return selection.StartRow > selection.EndRow;
            }
        }

        public int StartIndex
        {
            get { return selection.StartIndex; }
            set 
            { 
                selection.StartIndex = value;

                ObjectChangedEvent( this, EventArgs.Empty );
            }
        }

        public int StartRow
        {
            get { return selection.StartRow; }
            set 
            { 
                selection.StartRow = value;

                ObjectChangedEvent( this, EventArgs.Empty );
            }
        }

        public Color TextColor
        {
            get { return textColor; }
        }

        public event EventHandler ObjectChanged;
        public event EventHandler ObjectChanging;

        public DocumentSelection()
        {
            backColor = Brushes.DarkBlue;
            changeArgs = null;
            objectChanging = false;
            selection = new SSelection( 0, 0, 0, 0 );
            textColor = Color.White;
        }

        public DocumentSelection( int startRow, int startIndex, int endRow, int endIndex )
        {
            backColor = Brushes.DarkBlue;
            selection = new SSelection( startRow, startIndex, endRow, endIndex );
            textColor = Color.White;
        }

        public void BeginChange()
        {
            if( !objectChanging )
            {
                objectChanging = true;
                changeArgs = null;

                if( ObjectChanging != null ) ObjectChanging( this, EventArgs.Empty );
            }
        }

        public void Change( int startRow, int startIndex, int endRow, int endIndex )
        {
            selection.Change( startRow, startIndex, endRow, endIndex );

            ObjectChangedEvent( this, EventArgs.Empty );
        }

        public bool Contains( int row, int index )
        {
            if( ContainsRow( row ) )
            {
                if( selection.StartRow == selection.EndRow )
                    return selection.StartIndex <= index && index <= selection.EndIndex;

                if( row == selection.StartRow ) return selection.StartIndex <= index;
                else if( row == selection.EndRow ) return selection.EndIndex >= index;
                else return true;
           }
           
           return false;
        }

        public bool ContainsRow( int row )
        {
            if( Math.Min( selection.StartRow, selection.EndRow ) <= row &&
                row <= Math.Max( selection.StartRow, selection.EndRow ) ) return true;
            else return false;
        }

        public void EndChange( bool discardEvent )
        {
            if( objectChanging )
            {
                objectChanging = false;

                if( !discardEvent && changeArgs != null ) ObjectChangedEvent( this, changeArgs ); 
            }
        }

        public SSelection GetCurrent() 
        {
            return selection;
        }
        public SSelection GetOrdered() 
        {
            return selection.GetOrdered();
        }

        public Delegate[] GetReferences()
        {
            return ObjectChanged.GetInvocationList();
        }

        private void ObjectChangedEvent( object sender, EventArgs eventArgs )
        {
            if( objectChanging ) 
            {
                changeArgs = eventArgs;
                return;
            }

            if( ObjectChanged != null ) ObjectChanged( sender, eventArgs );
        }
    }
}
