﻿using System;
using System.Collections.Generic;
using System.Text;
using Editor.Collections;

namespace Editor.Document
{
    public sealed class RowCollection : ISimpleList<IDocumentRow>, IChangeHandler
    {
        private EventArgs changeArgs;
        private bool objectChanging;
        private GapBuffer<IDocumentRow> rowBuffer;

        public bool Changing
        {
            get { return objectChanging; }
        }

        public IDocumentRow this[int index]
        {
            get { return rowBuffer[index]; }
        }

        public int ItemCount
        {
            get { return rowBuffer.Length; }
        }

        public event EventHandler ObjectChanged;
        public event EventHandler ObjectChanging;
        public event EventHandler RowAdded;
        public event EventHandler RowChanged;
        public event EventHandler RowRemoved;

        public RowCollection()
        {
            changeArgs = null;
            objectChanging = false;
            rowBuffer = new GapBuffer<IDocumentRow>();
        }

        public void Add( IDocumentRow item )
        {
            rowBuffer.Insert( rowBuffer.Length, item );
        }

        public void BeginChange()
        {
            if( !objectChanging )
            {
                objectChanging = true;
                changeArgs = null;

                if( ObjectChanging != null ) ObjectChanging( this, EventArgs.Empty );
            }
        }

        public void Clear()
        {
            Queue<IDocumentRow> rowQueue = new Queue<IDocumentRow>();

            for( int i = 0; i < rowBuffer.Length; i++ )
                rowQueue.Enqueue( rowBuffer[i] );

            rowBuffer.SetBuffer( new IDocumentRow[0] );

            IDocumentRow documentRow;

            BeginChange();

            while( rowQueue.Count > 0 ) 
            {
                documentRow = rowQueue.Dequeue();

                documentRow.ObjectChanged -= OnRowChanged;

                RowRemovedEvent( documentRow, EventArgs.Empty );
            }

            EndChange( false );
        }

        public void EndChange( bool discardEvent )
        {
            if( objectChanging )
            {
                objectChanging = false;

                if( !discardEvent && changeArgs != null ) ObjectChangedEvent( this, changeArgs ); 
            }
        }

        public Delegate[] GetReferences()
        {
            return ObjectChanged.GetInvocationList();
        }

        public int IndexOf( IDocumentRow item )
        {
            for( int i = 0; i < rowBuffer.Length; i++ )
                if( object.ReferenceEquals( rowBuffer[i], item ) ) return i;
            return -1;
        }

        public int IndexOf( IDocumentRow item, int startIndex )
        {
            for( int i = startIndex; i < rowBuffer.Length; i++ )
                if( object.ReferenceEquals( rowBuffer[i], item ) ) return i;
            return -1;
        }

        public void Insert( int index, IDocumentRow item )
        {
            rowBuffer.Insert( index, item );

            item.ObjectChanged += OnRowChanged;

            RowAddedEvent( item, EventArgs.Empty );
        }
        
        public void InsertRange( int index, IDocumentRow[] items )
        {
            rowBuffer.Insert( index, items );
            {
                for( int i = 0; i < items.Length; i++ )
                {
                    items[i].ObjectChanged += OnRowChanged;
                    
                    RowAddedEvent( items[i], EventArgs.Empty );
                 }
            }

            ObjectChangedEvent( this, EventArgs.Empty );
        }

        public bool Remove( IDocumentRow item )
        {
            int index = IndexOf( item );

            if( index == -1 ) return false;
            else
            {
                rowBuffer[index].ObjectChanged -= OnRowChanged;

                rowBuffer.Remove( index, 1 );

                RowRemovedEvent( rowBuffer[index], EventArgs.Empty );

                return true;
            }
        }

        public void RemoveAt( int index )
        {
            rowBuffer.Remove( index, 1 );

            rowBuffer[index].ObjectChanged -= OnRowChanged;

            RowRemovedEvent( rowBuffer[index], EventArgs.Empty );
        }

        public void RemoveRange( int index, int length )
        {
            Queue<IDocumentRow> rowQueue = new Queue<IDocumentRow>();

            for( int i = index; i < index + length; i++ )
                rowQueue.Enqueue( rowBuffer[i] );

            rowBuffer.Remove( index, length );

            IDocumentRow documentRow;

            while( rowQueue.Count > 0 ) 
            {
                documentRow = rowQueue.Dequeue();

                documentRow.ObjectChanged -= OnRowChanged;

                RowRemovedEvent( documentRow, EventArgs.Empty );
            }
        }

        private void ObjectChangedEvent( object sender, EventArgs eventArgs )
        {
            if( objectChanging ) 
            {
                changeArgs = eventArgs;
                return;
            }

            if( ObjectChanged != null ) ObjectChanged( sender, eventArgs );
        }

        private void OnRowChanged( object sender, EventArgs eventArgs )
        {
            if( RowChanged != null ) RowChanged( sender, eventArgs );

            ObjectChangedEvent( sender, eventArgs );
        }

        private void RowAddedEvent( object sender, EventArgs eventArgs )
        {
            if( RowAdded != null ) RowAdded( sender, eventArgs );

            ObjectChangedEvent( sender, eventArgs );
        }

        private void RowRemovedEvent( object sender, EventArgs eventArgs )
        {
            if( RowRemoved != null ) RowRemoved( sender, eventArgs );

            ObjectChangedEvent( sender, eventArgs );
        }
    }
}
