﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Editor.Text;

namespace Editor.View
{
    public sealed class CharacterSegment : ISegment
    {
        private int indent;
        private int startIndex;
        private string characters;
        private IStyle style;
        private IView view;
        private int width;

        public int EndIndex
        {
            get { return startIndex + characters.Length - 1; }
        }

        public int Length
        {
            get { return characters.Length; }
        }

        public int Indent
        {
            get { return indent; }
        }

        public int StartIndex
        {
            get { return startIndex; }
        }

        public IStyle Style
        {
            get { return style; }
        }

        public int Width
        {
            get { return width; }
        }

        public CharacterSegment( IView view, string text, IStyle style, int startIndex, int indent, int width )
        {  
            this.characters = text;
            this.indent = indent;
            this.startIndex = startIndex;
            this.style = style;
            this.view = view;
            this.width = width;
        }

        public void Draw( int offset, bool selected, int selectionAscent, int selectionDescent )
        {
            Point location = new Point( indent, offset - style.GetAscent( view ) );

            if( selected )
            {
                view.Graphics.FillRectangle( view.Document.Selection.BackColor, indent, 
                    offset - selectionAscent, width, selectionAscent + selectionDescent );

                view.DrawString( characters, style.Font, view.Document.Selection.TextColor, location );
            }
            else
            {
                view.DrawString( characters, style.Font, style.TextColor,
                    style.BackColor, location );          
            }
        }

        public void Draw( int offset, int selectionStart, int selectionEnd,
            int selectionAscent, int selectionDescent )
        {
            Point location = new Point( indent, offset - style.GetAscent( view ) );
            Size size = Size.Empty;
            string substring = string.Empty;
            
            if( startIndex < selectionStart )
            {
                substring = characters.Substring( 0, selectionStart - startIndex );

                view.DrawString( substring, style.Font, style.TextColor,
                    style.BackColor, location );

                size = view.MeasureString( substring, style.Font );
                location.X += size.Width;
            }

            substring = characters.Substring( selectionStart - startIndex, 
                selectionEnd - selectionStart + 1 );

            size = view.MeasureString( substring, style.Font );

            view.Graphics.FillRectangle( view.Document.Selection.BackColor, location.X, 
                offset - selectionAscent, size.Width, selectionAscent + selectionDescent );

            view.DrawString( substring, style.Font, view.Document.Selection.TextColor, location );

            location.X += size.Width;

            if( selectionEnd < startIndex + characters.Length - 1 )
            {
                substring = characters.Substring( selectionEnd - startIndex + 1 );

                view.DrawString( substring, style.Font, style.TextColor,
                    style.BackColor, location );

                size = view.MeasureString( substring, style.Font );
            }
        }

        public int GetSelectionIndex( int x )
        {
            if( indent <= x && x <= indent + width )
            {
                StringBuilder buffer = new StringBuilder();
                Size currentSize = Size.Empty;

                x -= indent;

                for( int i = 0; i < characters.Length; i++ )
                {
                    buffer.Append( characters[i] );

                    currentSize = view.MeasureString( buffer.ToString(), style.Font );

                    if( x < currentSize.Width ) return startIndex + i;
                }
            }
            
            return -1;
        }

        public Size Measure( int index, int length )
        {
            if( length == 0 ) return Size.Empty;

            return view.MeasureString( characters.Substring( startIndex - index, length ),
                style.Font );
        }
    }
}
