﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Editor.Text;

namespace Editor.View
{
    public sealed class ViewLine : IViewLine
    {
        private int ascent;
        private int descent;
        private int height;
        private List<ISegment> segments;
        private IViewRow viewRow;

        public ISegment this[int index]
        {
            get { return segments[index]; }
            set
            {
                if( segments[index].Style.Equals( value.Style ) )
                    segments[index] = value;
                else
                {
                    segments[index] = value;

                    Update();
                }
            }
        }

        public int Height
        {
            get 
            { 
                if( height == 0 ) 
                    return viewRow.View.StyleTable[(short)EDefaultStyles.Normal].GetHeight( viewRow.View );

                return height;
            }
        }

        public int ItemCount
        {
            get { return segments.Count; }
        }

        public IViewRow ViewRow
        {
            get { return viewRow; }
        }

        public ViewLine( IViewRow viewRow )
        {
            ascent = 0;
            descent = 0;
            height = 0;
            segments = new List<ISegment>();
            this.viewRow = viewRow;
        }

        public void Add( ISegment item )
        {
            segments.Add( item );

            ascent = Math.Max( ascent, item.Style.GetAscent( viewRow.View ) );
            descent = Math.Max( descent, item.Style.GetDescent( viewRow.View ) );
            height = Math.Max( height, item.Style.GetHeight( viewRow.View ) );
        }

        public void Clear()
        {
            segments.Clear();

            ascent = 0;
            descent = 0;
            height = 0;
        }

        public void Draw( int offset, bool selected )
        {
            int lineBase = offset + ascent;

            for( int i = 0; i < segments.Count; i++ )
            {
                segments[i].Draw( lineBase, selected, ascent, descent );
            }      
        }

        public void Draw( int offset, int selectionStart )
        {
            int lineBase = offset + ascent;
            
            for( int i = 0; i < segments.Count; i++ )
            {
                if( segments[i].EndIndex < selectionStart )
                    segments[i].Draw( lineBase, false, ascent, descent );
                else 
                {
                    if( segments[i].StartIndex <= selectionStart )
                        segments[i].Draw( lineBase, selectionStart, segments[i].EndIndex, ascent, descent );
                    else segments[i].Draw( lineBase, true, ascent, descent );
                }
            }   
        }

        public void Draw( int offset, int selectionStart, int selectionEnd )
        {
            int lineBase = offset + ascent;
            int startIndex = 0;
            int endIndex = 0;

            for( int i = 0; i < segments.Count; i++ )
            {
                startIndex = Math.Max( segments[i].StartIndex, selectionStart );
                endIndex = Math.Min( segments[i].EndIndex, selectionEnd );

                if( startIndex > endIndex ) segments[i].Draw( lineBase, false, ascent, descent );
                else if ( startIndex == segments[i].StartIndex && endIndex == segments[i].EndIndex )
                    segments[i].Draw( lineBase, true, ascent, descent );
                else segments[i].Draw( lineBase, startIndex, endIndex, ascent, descent );
            }      
        }

        public void Insert( int index, ISegment item )
        {
            segments.Insert( index, item );

            ascent = Math.Max( ascent, item.Style.GetAscent( viewRow.View ) );
            descent = Math.Max( descent, item.Style.GetDescent( viewRow.View ) );
            height = Math.Max( height, item.Style.GetHeight( viewRow.View ) );
        }

        private void Update()
        {
            ascent = 0;
            descent = 0;
            height = 0;

            for( int i = 0; i < segments.Count; i++ )
            {
                ascent = Math.Max( ascent, segments[i].Style.GetAscent( viewRow.View ) );
                descent = Math.Max( descent, segments[i].Style.GetDescent( viewRow.View ) );
                height = Math.Max( height, segments[i].Style.GetHeight( viewRow.View ) );
            }       
        }
    }
}
